@@ -4,15 +4,16 @@ import sys
 import zmq
 
 from .selector import ZmqSelector
-from .core import ZmqEventLoop, ZmqEventLoopPolicy
+from .core import ZmqEventLoop, ZmqEventLoopPolicy, create_zmq_connection
 from .interface import ZmqTransport, ZmqProtocol
 
 
 __all__ = ('ZmqSelector', 'ZmqEventLoop', 'ZmqEventLoopPolicy',
            'ZmqTransport', 'ZmqProtocol',
+           'create_zmq_connection',
            'version_info', 'version')
 
-__version__ = '0.4.1'
+__version__ = '0.5.0'
 
 version = __version__ + ' , Python ' + sys.version
 
@@ -48,4 +49,5 @@ if zmq.zmq_version_info()[0] < 3:  # pragma no cover
 
 
 # make pyflakes happy
-(ZmqSelector, ZmqEventLoop, ZmqEventLoopPolicy, ZmqTransport, ZmqProtocol)
+(ZmqSelector, ZmqEventLoop, ZmqEventLoopPolicy, ZmqTransport, ZmqProtocol,
+ create_zmq_connection)
