@@ -3,6 +3,8 @@ import zmq
 
 from functools import partial
 
+from aiozmq import create_zmq_connection
+
 from .log import logger
 
 from .base import (
@@ -37,9 +39,9 @@ def connect_pipeline(*, connect=None, bind=None, loop=None,
     if loop is None:
         loop = asyncio.get_event_loop()
 
-    transp, proto = yield from loop.create_zmq_connection(
+    transp, proto = yield from create_zmq_connection(
         lambda: _ClientProtocol(loop, translation_table=translation_table),
-        zmq.PUSH, connect=connect, bind=bind)
+        zmq.PUSH, connect=connect, bind=bind, loop=loop)
     return PipelineClient(loop, proto)
 
 
@@ -72,12 +74,12 @@ def serve_pipeline(handler, *, connect=None, bind=None, loop=None,
     if loop is None:
         loop = asyncio.get_event_loop()
 
-    trans, proto = yield from loop.create_zmq_connection(
+    trans, proto = yield from create_zmq_connection(
         lambda: _ServerProtocol(loop, handler,
                                 translation_table=translation_table,
                                 log_exceptions=log_exceptions,
                                 exclude_log_exceptions=exclude_log_exceptions),
-        zmq.PULL, connect=connect, bind=bind)
+        zmq.PULL, connect=connect, bind=bind, loop=loop)
     return Service(loop, proto)
 
 
