@@ -13,6 +13,8 @@ import zmq
 from collections import ChainMap
 from functools import partial
 
+from aiozmq import create_zmq_connection
+
 from .log import logger
 
 from .base import (
@@ -63,10 +65,10 @@ def connect_rpc(*, connect=None, bind=None, loop=None,
     if loop is None:
         loop = asyncio.get_event_loop()
 
-    transp, proto = yield from loop.create_zmq_connection(
+    transp, proto = yield from create_zmq_connection(
         lambda: _ClientProtocol(loop, error_table=error_table,
                                 translation_table=translation_table),
-        zmq.DEALER, connect=connect, bind=bind)
+        zmq.DEALER, connect=connect, bind=bind, loop=loop)
     return RPCClient(loop, proto, timeout=timeout)
 
 
@@ -99,12 +101,12 @@ def serve_rpc(handler, *, connect=None, bind=None, loop=None,
     if loop is None:
         loop = asyncio.get_event_loop()
 
-    transp, proto = yield from loop.create_zmq_connection(
+    transp, proto = yield from create_zmq_connection(
         lambda: _ServerProtocol(loop, handler,
                                 translation_table=translation_table,
                                 log_exceptions=log_exceptions,
                                 exclude_log_exceptions=exclude_log_exceptions),
-        zmq.ROUTER, connect=connect, bind=bind)
+        zmq.ROUTER, connect=connect, bind=bind, loop=loop)
     return Service(loop, proto)
 
 
@@ -228,7 +230,7 @@ class _ServerProtocol(_BaseServerProtocol):
 
     def msg_received(self, data):
         try:
-            peer, header, bname, bargs, bkwargs = data
+            *pre, header, bname, bargs, bkwargs = data
             pid, rnd, req_id, timestamp = self.REQ.unpack(header)
 
             name = bname.decode('utf-8')
@@ -243,7 +245,7 @@ class _ServerProtocol(_BaseServerProtocol):
         except (NotFoundError, ParametersError) as exc:
             fut = asyncio.Future(loop=self.loop)
             fut.add_done_callback(partial(self.process_call_result,
-                                          req_id=req_id, peer=peer,
+                                          req_id=req_id, pre=pre,
                                           name=name, args=args, kwargs=kwargs))
             fut.set_exception(exc)
         else:
@@ -257,13 +259,13 @@ class _ServerProtocol(_BaseServerProtocol):
                 except Exception as exc:
                     fut.set_exception(exc)
             fut.add_done_callback(partial(self.process_call_result,
-                                          req_id=req_id, peer=peer,
+                                          req_id=req_id, pre=pre,
                                           return_annotation=ret_ann,
                                           name=name,
                                           args=args,
                                           kwargs=kwargs))
 
-    def process_call_result(self, fut, *, req_id, peer, name,
+    def process_call_result(self, fut, *, req_id, pre, name,
                             args, kwargs,
                             return_annotation=None):
         self.pending_waiters.discard(fut)
@@ -276,7 +278,7 @@ class _ServerProtocol(_BaseServerProtocol):
                 ret = return_annotation(ret)
             prefix = self.prefix + self.RESP_SUFFIX.pack(req_id,
                                                          time.time(), False)
-            self.transport.write([peer, prefix, self.packer.packb(ret)])
+            self.transport.write(pre + [prefix, self.packer.packb(ret)])
         except asyncio.CancelledError:
             return
         except Exception as exc:
@@ -285,4 +287,4 @@ class _ServerProtocol(_BaseServerProtocol):
             exc_type = exc.__class__
             exc_info = (exc_type.__module__ + '.' + exc_type.__name__,
                         exc.args, repr(exc))
-            self.transport.write([peer, prefix, self.packer.packb(exc_info)])
+            self.transport.write(pre + [prefix, self.packer.packb(exc_info)])
