@@ -21,21 +21,11 @@ class _MethodCall:
         if not self._names:
             raise ValueError('RPC method name is empty')
         fut = self._proto.call('.'.join(self._names), args, kwargs)
-        return _Waiter(fut, timeout=self._timeout, loop=self._proto.loop)
-
-
-class _Waiter:
-
-    __slots__ = ('_fut', '_timeout', '_loop')
-
-    def __init__(self, fut, *, timeout=None, loop=None):
-        self._fut = fut
-        self._timeout = timeout
-        self._loop = loop
-
-    def __iter__(self):
-        return (yield from asyncio.wait_for(self._fut, self._timeout,
-                                            loop=self._loop))
+        loop = self._proto.loop
+        return asyncio.Task(asyncio.wait_for(fut,
+                                             timeout=self._timeout,
+                                             loop=loop),
+                            loop=loop)
 
 
 def _fill_error_table():
